@@ -50,10 +50,17 @@ QT_END_NAMESPACE
 typedef QtSingleCoreApplication BaseApplication;
 #endif
 
+#include "base/utils/misc.h"
+
 #ifndef DISABLE_WEBUI
 class WebUI;
 #endif
 
+namespace BitTorrent
+{
+    class TorrentHandle;
+}
+
 class Application : public BaseApplication
 {
     Q_OBJECT
@@ -77,6 +84,8 @@ protected:
 
 private slots:
     void processMessage(const QString &message);
+    void torrentFinished(BitTorrent::TorrentHandle *const torrent);
+    void allTorrentsFinished();
     void cleanup();
 #if (!defined(DISABLE_GUI) && defined(Q_OS_WIN))
     void shutdownCleanup(QSessionManager &manager);
@@ -87,6 +96,7 @@ private:
 
 #ifndef DISABLE_GUI
     QPointer<MainWindow> m_window;
+    ShutdownAction m_shutdownAct;
 #endif
 
 #ifndef DISABLE_WEBUI
@@ -99,6 +109,7 @@ private:
 
     void initializeTranslation();
     void processParams(const QStringList &params);
+    void sendNotificationEmail(BitTorrent::TorrentHandle *const torrent);
 };
 
 #endif // APPLICATION_H
