@@ -34,7 +34,7 @@
 #include <QShortcut>
 #include <QWidget>
 #include "ui_propertieswidget.h"
-#include "qtorrenthandle.h"
+#include "base/bittorrent/torrenthandle.h"
 
 
 class TransferListWidget;
@@ -43,6 +43,7 @@ class PropListDelegate;
 class torrent_file;
 class PeerListWidget;
 class TrackerList;
+class SpeedWidget;
 class MainWindow;
 class DownloadedPiecesBar;
 class PieceAvailabilityBar;
@@ -64,23 +65,19 @@ public:
 public:
   PropertiesWidget(QWidget *parent, MainWindow* main_window, TransferListWidget *transferList);
   ~PropertiesWidget();
-  QTorrentHandle getCurrentTorrent() const;
+  BitTorrent::TorrentHandle *getCurrentTorrent() const;
   TrackerList* getTrackerList() const { return trackerList; }
   PeerListWidget* getPeerList() const { return peersList; }
   QTreeView* getFilesList() const { return filesList; }
-
-signals:
-  void trackersAdded(const QStringList &trackers, const QString &hash);
-  void trackersRemoved(const QStringList &trackers, const QString &hash);
-  void trackerlessChange(bool trackerless, const QString &hash);
+  SpeedWidget* getSpeedWidget() const { return speedWidget; }
 
 protected:
   QPushButton* getButtonFromIndex(int index);
   bool applyPriorities();
 
 protected slots:
-  void loadTorrentInfos(const QTorrentHandle &h);
-  void updateTorrentInfos(const QTorrentHandle &h);
+  void loadTorrentInfos(BitTorrent::TorrentHandle *const torrent);
+  void updateTorrentInfos(BitTorrent::TorrentHandle *const torrent);
   void loadUrlSeeds();
   void askWebSeed();
   void deleteSelectedUrlSeeds();
@@ -102,8 +99,7 @@ public slots:
   void saveSettings();
   void reloadPreferences();
   void openDoubleClickedFile(const QModelIndex &);
-  void updateSavePath(const QTorrentHandle& h);
-  void loadTrackers(const QTorrentHandle &handle);
+  void loadTrackers(BitTorrent::TorrentHandle *const torrent);
 
 private:
   void openFile(const QModelIndex &index);
@@ -112,18 +108,19 @@ private:
 private:
   TransferListWidget *transferList;
   MainWindow *main_window;
-  QTorrentHandle h;
+  BitTorrent::TorrentHandle *m_torrent;
   QTimer *refreshTimer;
   SlideState state;
   TorrentContentFilterModel *PropListModel;
   PropListDelegate *PropDelegate;
   PeerListWidget *peersList;
   TrackerList *trackerList;
+  SpeedWidget *speedWidget;
   QList<int> slideSizes;
   DownloadedPiecesBar *downloaded_pieces;
   PieceAvailabilityBar *pieces_availability;
   PropTabBar *m_tabBar;
-  LineEdit *m_contentFilerLine;
+  LineEdit *m_contentFilterLine;
   QShortcut *editHotkeyFile;
   QShortcut *editHotkeyWeb;
   QShortcut *deleteHotkeyWeb;
@@ -131,6 +128,7 @@ private:
 
 private slots:
   void filterText(const QString& filter);
+  void updateSavePath(BitTorrent::TorrentHandle *const torrent);
 };
 
 #endif // PROPERTIESWIDGET_H
