@@ -39,9 +39,10 @@
 
 #include "rss_imp.h"
 #include "feedlistwidget.h"
-#include "qbtsession.h"
+#include "base/bittorrent/session.h"
+#include "base/net/downloadmanager.h"
 #include "cookiesdlg.h"
-#include "preferences.h"
+#include "base/preferences.h"
 #include "rsssettingsdlg.h"
 #include "rssmanager.h"
 #include "rssfolder.h"
@@ -49,8 +50,9 @@
 #include "rssparser.h"
 #include "rssfeed.h"
 #include "automatedrssdownloader.h"
-#include "iconprovider.h"
+#include "guiiconprovider.h"
 #include "autoexpandabledialog.h"
+#include "addnewtorrentdialog.h"
 
 namespace Article
 {
@@ -110,20 +112,31 @@ void RSSImp::displayItemsListMenu(const QPoint&)
 {
     QMenu myItemListMenu(this);
     QList<QListWidgetItem*> selectedItems = listArticles->selectedItems();
-    if (selectedItems.size() > 0) {
-        bool has_attachment = false;
-        foreach (const QListWidgetItem* item, selectedItems) {
-            if (m_feedList->getRSSItemFromUrl(item->data(Article::FeedUrlRole).toString())
-                ->getItem(item->data(Article::IdRole).toString())->hasAttachment()) {
-                has_attachment = true;
-                break;
-            }
-        }
-        if (has_attachment)
-            myItemListMenu.addAction(actionDownload_torrent);
-        myItemListMenu.addAction(actionOpen_news_URL);
+    if (selectedItems.size() <= 0)
+        return;
+
+    bool hasTorrent = false;
+    bool hasLink = false;
+    foreach (const QListWidgetItem* item, selectedItems) {
+        if (!item) continue;
+        RssFeedPtr feed = m_feedList->getRSSItemFromUrl(item->data(Article::FeedUrlRole).toString());
+        if (!feed) continue;
+        RssArticlePtr article = feed->getItem(item->data(Article::IdRole).toString());
+        if (!article) continue;
+
+        if (!article->torrentUrl().isEmpty())
+            hasTorrent = true;
+        if (!article->link().isEmpty())
+            hasLink = true;
+        if (hasTorrent && hasLink)
+            break;
     }
-    myItemListMenu.exec(QCursor::pos());
+    if (hasTorrent)
+        myItemListMenu.addAction(actionDownload_torrent);
+    if (hasLink)
+        myItemListMenu.addAction(actionOpen_news_URL);
+    if (hasTorrent || hasLink)
+        myItemListMenu.exec(QCursor::pos());
 }
 
 void RSSImp::on_actionManage_cookies_triggered()
@@ -140,6 +153,7 @@ void RSSImp::on_actionManage_cookies_triggered()
     if (ok) {
         qDebug() << "Settings cookies for host name: " << feed_hostname;
         pref->setHostNameCookies(feed_hostname, raw_cookies);
+        Net::DownloadManager::instance()->setCookiesFromUrl(pref->getHostNameQNetworkCookies(feed_hostname), feed_hostname);
     }
 }
 
@@ -225,7 +239,6 @@ void RSSImp::on_newFeedButton_clicked()
     // Notify TreeWidget
     m_feedList->itemAdded(item, stream);
 
-    stream->refresh();
     m_rssManager->saveStreamList();
 }
 
@@ -235,29 +248,16 @@ void RSSImp::deleteSelectedItems()
     QList<QTreeWidgetItem*> selectedItems = m_feedList->selectedItems();
     if (selectedItems.isEmpty())
         return;
+    if ((selectedItems.size() == 1) && (selectedItems.first() == m_feedList->stickyUnreadItem()))
+        return;
 
-    int ret;
-    if (selectedItems.size() > 1) {
-        ret = QMessageBox::question(this, tr("Are you sure? -- qBittorrent"), tr("Are you sure you want to delete these elements from the list?"),
-                                    tr("&Yes"), tr("&No"),
-                                    QString(), 0, 1);
-    }
-    else {
-        if (selectedItems.first() == m_feedList->stickyUnreadItem())
-            return;
-        ret = QMessageBox::question(this, tr("Are you sure? -- qBittorrent"), tr("Are you sure you want to delete this element from the list?"),
-                                    tr("&Yes"), tr("&No"),
-                                    QString(), 0, 1);
-    }
-    if (ret)
+    QMessageBox::StandardButton answer = QMessageBox::question(this, tr("Deletion confirmation"),
+        tr("Are you sure you want to delete the selected RSS feeds?"),
+        QMessageBox::Yes|QMessageBox::No, QMessageBox::No);
+    if (answer == QMessageBox::No)
         return;
 
     foreach (QTreeWidgetItem* item, selectedItems) {
-        if (m_feedList->currentFeed() == item) {
-            textBrowser->clear();
-            m_currentArticle = 0;
-            listArticles->clear();
-        }
         if (item == m_feedList->stickyUnreadItem())
             continue;
         RssFilePtr rss_item = m_feedList->getRSSItem(item);
@@ -280,6 +280,9 @@ void RSSImp::deleteSelectedItems()
     m_rssManager->saveStreamList();
     // Update Unread items
     updateItemInfos(m_feedList->stickyUnreadItem());
+    if (m_feedList->currentItem() == m_feedList->stickyUnreadItem())
+        populateArticleList(m_feedList->stickyUnreadItem());
+
 }
 
 void RSSImp::loadFoldersOpenState()
@@ -334,40 +337,57 @@ void RSSImp::refreshAllFeeds()
 void RSSImp::downloadSelectedTorrents()
 {
     QList<QListWidgetItem*> selected_items = listArticles->selectedItems();
-    foreach (const QListWidgetItem* item, selected_items) {
+    if (selected_items.size() <= 0)
+        return;
+    foreach (QListWidgetItem* item, selected_items) {
         if (!item) continue;
         RssFeedPtr feed = m_feedList->getRSSItemFromUrl(item->data(Article::FeedUrlRole).toString());
         if (!feed) continue;
         RssArticlePtr article = feed->getItem(item->data(Article::IdRole).toString());
         if (!article) continue;
 
-        QString torrentLink = article->torrentUrl();
-        // Check if it is a magnet link
-        if (torrentLink.startsWith("magnet:", Qt::CaseInsensitive)) {
-            QBtSession::instance()->addMagnetInteractive(torrentLink);
-        }
-        else {
-            // Load possible cookies
-            QString feed_url = m_feedList->getItemID(m_feedList->selectedItems().first());
-            QString feed_hostname = QUrl::fromEncoded(feed_url.toUtf8()).host();
-            QList<QNetworkCookie> cookies = Preferences::instance()->getHostNameQNetworkCookies(feed_hostname);
-            qDebug("Loaded %d cookies for RSS item\n", cookies.size());
-            QBtSession::instance()->downloadFromUrl(torrentLink, cookies);
-        }
+        // Mark as read
+        article->markAsRead();
+        item->setData(Article::ColorRole, QVariant(QColor("grey")));
+        item->setData(Article::IconRole, QVariant(QIcon(":/icons/sphere.png")));
+
+        if (article->torrentUrl().isEmpty())
+            continue;
+        if (Preferences::instance()->useAdditionDialog())
+            AddNewTorrentDialog::show(article->torrentUrl());
+        else
+            BitTorrent::Session::instance()->addTorrent(article->torrentUrl());
     }
+    // Decrement feed nb unread news
+    updateItemInfos(m_feedList->stickyUnreadItem());
+    updateItemInfos(m_feedList->getTreeItemFromUrl(selected_items.first()->data(Article::FeedUrlRole).toString()));
 }
 
 // open the url of the selected RSS articles in the Web browser
 void RSSImp::openSelectedArticlesUrls()
 {
     QList<QListWidgetItem *> selected_items = listArticles->selectedItems();
-    foreach (const QListWidgetItem* item, selected_items) {
-        RssArticlePtr news =  m_feedList->getRSSItemFromUrl(item->data(Article::FeedUrlRole).toString())
-                             ->getItem(item->data(Article::IdRole).toString());
-        const QString link = news->link();
+    if (selected_items.size() <= 0)
+        return;
+    foreach (QListWidgetItem* item, selected_items) {
+        if (!item) continue;
+        RssFeedPtr feed = m_feedList->getRSSItemFromUrl(item->data(Article::FeedUrlRole).toString());
+        if (!feed) continue;
+        RssArticlePtr article = feed->getItem(item->data(Article::IdRole).toString());
+        if (!article) continue;
+
+        // Mark as read
+        article->markAsRead();
+        item->setData(Article::ColorRole, QVariant(QColor("grey")));
+        item->setData(Article::IconRole, QVariant(QIcon(":/icons/sphere.png")));
+
+        const QString link = article->link();
         if (!link.isEmpty())
             QDesktopServices::openUrl(QUrl(link));
     }
+    // Decrement feed nb unread news
+    updateItemInfos(m_feedList->stickyUnreadItem());
+    updateItemInfos(m_feedList->getTreeItemFromUrl(selected_items.first()->data(Article::FeedUrlRole).toString()));
 }
 
 //right-click on stream : give it an alias
@@ -544,23 +564,15 @@ void RSSImp::refreshTextBrowser()
 {
     QList<QListWidgetItem*> selection = listArticles->selectedItems();
     if (selection.empty()) return;
-    Q_ASSERT(selection.size() == 1);
     QListWidgetItem *item = selection.first();
     Q_ASSERT(item);
     if (item == m_currentArticle) return;
-    // Stop displaying previous news if necessary
-    if (m_feedList->currentFeed() == m_feedList->stickyUnreadItem()) {
-        if (m_currentArticle) {
-            disconnect(listArticles, SIGNAL(itemSelectionChanged()), this, SLOT(refreshTextBrowser()));
-            listArticles->removeItemWidget(m_currentArticle);
-            Q_ASSERT(m_currentArticle);
-            delete m_currentArticle;
-            connect(listArticles, SIGNAL(itemSelectionChanged()), this, SLOT(refreshTextBrowser()));
-        }
-        m_currentArticle = item;
-    }
-    RssFeedPtr stream = m_feedList->getRSSItemFromUrl(item->data(Article::FeedUrlRole).toString());
-    RssArticlePtr article = stream->getItem(item->data(Article::IdRole).toString());
+    m_currentArticle = item;
+
+    RssFeedPtr feed = m_feedList->getRSSItemFromUrl(item->data(Article::FeedUrlRole).toString());
+    if (!feed) return;
+    RssArticlePtr article = feed->getItem(item->data(Article::IdRole).toString());
+    if (!article) return;
     QString html;
     html += "<div style='border: 2px solid red; margin-left: 5px; margin-right: 5px; margin-bottom: 5px;'>";
     html += "<div style='background-color: #678db2; font-weight: bold; color: #fff;'>" + article->title() + "</div>";
@@ -622,11 +634,11 @@ void RSSImp::saveSlidersPosition()
 void RSSImp::restoreSlidersPosition()
 {
     const Preferences* const pref = Preferences::instance();
-    QByteArray pos_h = pref->getRssHSplitterState();
-    if (!pos_h.isNull())
+    const QByteArray pos_h = pref->getRssHSplitterState();
+    if (!pos_h.isEmpty())
         splitter_h->restoreState(pos_h);
-    QByteArray pos_v = pref->getRssVSplitterState();
-    if (!pos_v.isNull())
+    const QByteArray pos_v = pref->getRssVSplitterState();
+    if (!pos_v.isEmpty())
         splitter_v->restoreState(pos_v);
 }
 
@@ -681,14 +693,11 @@ void RSSImp::onFeedContentChanged(const QString& url)
     qDebug() << Q_FUNC_INFO << url;
     QTreeWidgetItem *item = m_feedList->getTreeItemFromUrl(url);
     // If the feed is selected, update the displayed news
-    if (m_feedList->currentItem() == item ) {
+    if (m_feedList->currentItem() == item)
         populateArticleList(item);
-    }
-    else {
-        // Update unread items
-        if (m_feedList->currentItem() == m_feedList->stickyUnreadItem())
-            populateArticleList(m_feedList->stickyUnreadItem());
-    }
+    // Update unread items
+    else if (m_feedList->currentItem() == m_feedList->stickyUnreadItem())
+        populateArticleList(m_feedList->stickyUnreadItem());
 }
 
 void RSSImp::updateRefreshInterval(uint val)
@@ -702,27 +711,25 @@ RSSImp::RSSImp(QWidget *parent):
 {
     setupUi(this);
     // Icons
-    actionCopy_feed_URL->setIcon(IconProvider::instance()->getIcon("edit-copy"));
-    actionDelete->setIcon(IconProvider::instance()->getIcon("edit-delete"));
-    actionDownload_torrent->setIcon(IconProvider::instance()->getIcon("download"));
-    actionManage_cookies->setIcon(IconProvider::instance()->getIcon("preferences-web-browser-cookies"));
-    actionMark_items_read->setIcon(IconProvider::instance()->getIcon("mail-mark-read"));
-    actionNew_folder->setIcon(IconProvider::instance()->getIcon("folder-new"));
-    actionNew_subscription->setIcon(IconProvider::instance()->getIcon("list-add"));
-    actionOpen_news_URL->setIcon(IconProvider::instance()->getIcon("application-x-mswinurl"));
-    actionRename->setIcon(IconProvider::instance()->getIcon("edit-rename"));
-    actionUpdate->setIcon(IconProvider::instance()->getIcon("view-refresh"));
-    actionUpdate_all_feeds->setIcon(IconProvider::instance()->getIcon("view-refresh"));
-    newFeedButton->setIcon(IconProvider::instance()->getIcon("list-add"));
-    markReadButton->setIcon(IconProvider::instance()->getIcon("mail-mark-read"));
-    updateAllButton->setIcon(IconProvider::instance()->getIcon("view-refresh"));
-    rssDownloaderBtn->setIcon(IconProvider::instance()->getIcon("download"));
-    settingsButton->setIcon(IconProvider::instance()->getIcon("preferences-system"));
+    actionCopy_feed_URL->setIcon(GuiIconProvider::instance()->getIcon("edit-copy"));
+    actionDelete->setIcon(GuiIconProvider::instance()->getIcon("edit-delete"));
+    actionDownload_torrent->setIcon(GuiIconProvider::instance()->getIcon("download"));
+    actionManage_cookies->setIcon(GuiIconProvider::instance()->getIcon("preferences-web-browser-cookies"));
+    actionMark_items_read->setIcon(GuiIconProvider::instance()->getIcon("mail-mark-read"));
+    actionNew_folder->setIcon(GuiIconProvider::instance()->getIcon("folder-new"));
+    actionNew_subscription->setIcon(GuiIconProvider::instance()->getIcon("list-add"));
+    actionOpen_news_URL->setIcon(GuiIconProvider::instance()->getIcon("application-x-mswinurl"));
+    actionRename->setIcon(GuiIconProvider::instance()->getIcon("edit-rename"));
+    actionUpdate->setIcon(GuiIconProvider::instance()->getIcon("view-refresh"));
+    actionUpdate_all_feeds->setIcon(GuiIconProvider::instance()->getIcon("view-refresh"));
+    newFeedButton->setIcon(GuiIconProvider::instance()->getIcon("list-add"));
+    markReadButton->setIcon(GuiIconProvider::instance()->getIcon("mail-mark-read"));
+    updateAllButton->setIcon(GuiIconProvider::instance()->getIcon("view-refresh"));
+    rssDownloaderBtn->setIcon(GuiIconProvider::instance()->getIcon("download"));
+    settingsButton->setIcon(GuiIconProvider::instance()->getIcon("preferences-system"));
 
     m_feedList = new FeedListWidget(splitter_h, m_rssManager);
     splitter_h->insertWidget(0, m_feedList);
-    listArticles->setSelectionBehavior(QAbstractItemView::SelectItems);
-    listArticles->setSelectionMode(QAbstractItemView::SingleSelection);
     editHotkey = new QShortcut(QKeySequence("F2"), m_feedList, 0, 0, Qt::WidgetShortcut);
     connect(editHotkey, SIGNAL(activated()), SLOT(renameSelectedRssFile()));
     connect(m_feedList, SIGNAL(doubleClicked(QModelIndex)), SLOT(renameSelectedRssFile()));
