@@ -32,8 +32,9 @@
 #define TRANSFERLISTSORTMODEL_H
 
 #include <QSortFilterProxyModel>
-#include <QStringList>
-#include "torrentfilterenum.h"
+#include "base/torrentfilter.h"
+
+class QStringList;
 
 class TransferListSortModel: public QSortFilterProxyModel
 {
@@ -42,7 +43,7 @@ class TransferListSortModel: public QSortFilterProxyModel
 public:
     TransferListSortModel(QObject *parent = 0);
 
-    void setStatusFilter(const TorrentFilter::TorrentFilter &filter);
+    void setStatusFilter(TorrentFilter::Type filter);
     void setLabelFilter(const QString &label);
     void disableLabelFilter();
     void setTrackerFilter(const QStringList &hashes);
@@ -52,18 +53,10 @@ private:
     bool lessThan(const QModelIndex &left, const QModelIndex &right) const;
     bool lowerPositionThan(const QModelIndex &left, const QModelIndex &right) const;
     bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const;
-
-    bool matchStatusFilter(int sourceRow, const QModelIndex &sourceParent) const;
-    bool matchLabelFilter(int sourceRow, const QModelIndex &sourceParent) const;
-    bool matchTrackerFilter(int sourceRow, const QModelIndex &sourceParent) const;
+    bool matchFilter(int sourceRow, const QModelIndex &sourceParent) const;
 
 private:
-    TorrentFilter::TorrentFilter filter0;
-
-    bool labelFilterEnabled;
-    QString labelFilter;
-    bool trackerFilterEnabled;
-    QStringList trackerFilter;
+    TorrentFilter m_filter;
 };
 
 #endif // TRANSFERLISTSORTMODEL_H
