@@ -32,10 +32,12 @@
 #define TRANSFERLISTWIDGET_H
 
 #include <QTreeView>
-#include <libtorrent/version.hpp>
 
-class QBtSession;
-class QTorrentHandle;
+namespace BitTorrent
+{
+    class TorrentHandle;
+}
+
 class MainWindow;
 class TransferListDelegate;
 class TransferListSortModel;
@@ -52,14 +54,15 @@ class TransferListWidget: public QTreeView
     Q_OBJECT
 
 public:
-    TransferListWidget(QWidget *parent, MainWindow *main_window, QBtSession* BTSession);
+    TransferListWidget(QWidget *parent, MainWindow *main_window);
     ~TransferListWidget();
     TorrentModel* getSourceModel() const;
 
 public slots:
     void setSelectionLabel(QString label);
-    void setRefreshInterval(int t);
     void setSelectedTorrentsLocation();
+    void pauseAllTorrents();
+    void resumeAllTorrents();
     void startSelectedTorrents();
     void forceStartSelectedTorrents();
     void startVisibleTorrents();
@@ -83,7 +86,6 @@ public slots:
     void displayDLHoSMenu(const QPoint&);
     void applyNameFilter(const QString& name);
     void applyStatusFilter(int f);
-    void applyLabelFilterAll();
     void applyLabelFilter(QString label);
     void applyTrackerFilterAll();
     void applyTrackerFilter(const QStringList &hashes);
@@ -92,12 +94,10 @@ public slots:
     void renameSelectedTorrent();
 
 protected:
-    int getRowFromHash(QString hash) const;
-    QString getHashFromRow(int row) const;
     QModelIndex mapToSource(const QModelIndex &index) const;
     QModelIndex mapFromSource(const QModelIndex &index) const;
     bool loadSettings();
-    QStringList getSelectedTorrentsHashes() const;
+    QList<BitTorrent::TorrentHandle *> getSelectedTorrents() const;
 
 protected slots:
     void torrentDoubleClicked(const QModelIndex& index);
@@ -109,17 +109,13 @@ protected slots:
     void askNewLabelForSelection();
     void saveSettings();
 
-private:
-    bool openUrl(const QString& _path) const;
-
 signals:
-    void currentTorrentChanged(const QTorrentHandle &h);
+    void currentTorrentChanged(BitTorrent::TorrentHandle *const torrent);
 
 private:
     TransferListDelegate *listDelegate;
     TorrentModel *listModel;
     TransferListSortModel *nameFilterModel;
-    QBtSession* BTSession;
     MainWindow *main_window;
     QShortcut *editHotkey;
     QShortcut *deleteHotkey;
