@@ -29,13 +29,14 @@
  */
 
 #include "engineselectdlg.h"
-#include "downloadthread.h"
-#include "fs_utils.h"
-#include "misc.h"
+#include "base/net/downloadmanager.h"
+#include "base/net/downloadhandler.h"
+#include "base/utils/fs.h"
+#include "base/utils/misc.h"
 #include "ico.h"
 #include "searchengine.h"
 #include "pluginsource.h"
-#include "iconprovider.h"
+#include "guiiconprovider.h"
 #include "autoexpandabledialog.h"
 #include <QProcess>
 #include <QHeaderView>
@@ -45,27 +46,36 @@
 #include <QDropEvent>
 #include <QTemporaryFile>
 #include <QMimeData>
+#include <QClipboard>
+#ifdef QBT_USES_QT5
+#include <QTableView>
+#endif
 
 enum EngineColumns {ENGINE_NAME, ENGINE_VERSION, ENGINE_URL, ENGINE_STATE, ENGINE_ID};
 
 engineSelectDlg::engineSelectDlg(QWidget *parent, SupportedEngines *supported_engines)
     : QDialog(parent)
     , supported_engines(supported_engines)
-    , m_updateUrl(QString("https://raw.github.com/qbittorrent/qBittorrent/master/src/searchengine/") + (misc::pythonVersion() >= 3 ? "nova3" : "nova") + "/engines/")
+    , m_updateUrl(QString("https://raw.github.com/qbittorrent/qBittorrent/master/src/searchengine/") + (Utils::Misc::pythonVersion() >= 3 ? "nova3" : "nova") + "/engines/")
 {
   setupUi(this);
   setAttribute(Qt::WA_DeleteOnClose);
+#ifdef QBT_USES_QT5
+  // This hack fixes reordering of first column with Qt5.
+  // https://github.com/qtproject/qtbase/commit/e0fc088c0c8bc61dbcaf5928b24986cd61a22777
+  QTableView unused;
+  unused.setVerticalHeader(pluginsTree->header());
+  pluginsTree->header()->setParent(pluginsTree);
+  unused.setVerticalHeader(new QHeaderView(Qt::Horizontal));
+#endif
   pluginsTree->setRootIsDecorated(false);
   pluginsTree->header()->resizeSection(0, 160);
   pluginsTree->header()->resizeSection(1, 80);
   pluginsTree->header()->resizeSection(2, 200);
   pluginsTree->hideColumn(ENGINE_ID);
-  actionUninstall->setIcon(IconProvider::instance()->getIcon("list-remove"));
+  actionUninstall->setIcon(GuiIconProvider::instance()->getIcon("list-remove"));
   connect(actionEnable, SIGNAL(toggled(bool)), this, SLOT(enableSelection(bool)));
   connect(pluginsTree, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(displayContextMenu(const QPoint&)));
-  downloader = new DownloadThread(this);
-  connect(downloader, SIGNAL(downloadFinished(QString, QString)), this, SLOT(processDownloadedFile(QString, QString)));
-  connect(downloader, SIGNAL(downloadFailure(QString, QString)), this, SLOT(handleDownloadFailure(QString, QString)));
   loadSupportedSearchEngines();
   connect(supported_engines, SIGNAL(newSupportedEngine(QString)), this, SLOT(addNewEngine(QString)));
   connect(pluginsTree, SIGNAL(itemDoubleClicked(QTreeWidgetItem*, int)), this, SLOT(toggleEngineState(QTreeWidgetItem*, int)));
@@ -75,8 +85,6 @@ engineSelectDlg::engineSelectDlg(QWidget *parent, SupportedEngines *supported_en
 engineSelectDlg::~engineSelectDlg() {
   qDebug("Destroying engineSelectDlg");
   emit enginesChanged();
-  qDebug("Before deleting downloader");
-  delete downloader;
   qDebug("Engine plugins dialog destroyed");
 }
 
@@ -99,15 +107,15 @@ void engineSelectDlg::dropEvent(QDropEvent *event) {
   }
   foreach (QString file, files) {
     qDebug("dropped %s", qPrintable(file));
-    if (misc::isUrl(file)) {
+    if (Utils::Misc::isUrl(file)) {
       setCursor(QCursor(Qt::WaitCursor));
-      downloader->downloadUrl(file);
+      downloadFromUrl(file);
       continue;
     }
     if (file.endsWith(".py", Qt::CaseInsensitive)) {
       if (file.startsWith("file:", Qt::CaseInsensitive))
         file = QUrl(file).toLocalFile();
-      QString plugin_name = fsutils::fileName(file);
+      QString plugin_name = Utils::Fs::fileName(file);
       plugin_name.chop(3); // Remove extension
       installPlugin(file, plugin_name);
     }
@@ -128,7 +136,7 @@ void engineSelectDlg::dragEnterEvent(QDragEnterEvent *event) {
 void engineSelectDlg::on_updateButton_clicked() {
   // Download version file from update server on sourceforge
   setCursor(QCursor(Qt::WaitCursor));
-  downloader->downloadUrl(m_updateUrl + "versions.txt");
+  downloadFromUrl(m_updateUrl + "versions.txt");
 }
 
 void engineSelectDlg::toggleEngineState(QTreeWidgetItem *item, int) {
@@ -178,7 +186,7 @@ void engineSelectDlg::on_actionUninstall_triggered() {
     }else {
       // Proceed with uninstall
       // remove it from hard drive
-      QDir enginesFolder(fsutils::searchEngineLocation() + "/engines");
+      QDir enginesFolder(Utils::Fs::searchEngineLocation() + "/engines");
       QStringList filters;
       filters << id+".*";
       QStringList files = enginesFolder.entryList(filters, QDir::Files, QDir::Unsorted);
@@ -244,7 +252,7 @@ QTreeWidgetItem* engineSelectDlg::findItemWithID(QString id) {
 }
 
 bool engineSelectDlg::isUpdateNeeded(QString plugin_name, qreal new_version) const {
-  qreal old_version = SearchEngine::getPluginVersion(fsutils::searchEngineLocation() + "/engines/" + plugin_name + ".py");
+  qreal old_version = SearchEngine::getPluginVersion(Utils::Fs::searchEngineLocation() + "/engines/" + plugin_name + ".py");
   qDebug("IsUpdate needed? tobeinstalled: %.2f, alreadyinstalled: %.2f", new_version, old_version);
   return (new_version > old_version);
 }
@@ -252,20 +260,24 @@ bool engineSelectDlg::isUpdateNeeded(QString plugin_name, qreal new_version) con
 void engineSelectDlg::installPlugin(QString path, QString plugin_name) {
   qDebug("Asked to install plugin at %s", qPrintable(path));
   qreal new_version = SearchEngine::getPluginVersion(path);
+  if (new_version == 0.0) {
+    QMessageBox::warning(this, tr("Invalid plugin"), tr("The search engine plugin is invalid, please contact the author."));
+    return;
+  }
   qDebug("Version to be installed: %.2f", new_version);
   if (!isUpdateNeeded(plugin_name, new_version)) {
     qDebug("Apparently update is not needed, we have a more recent version");
-    QMessageBox::information(this, tr("Search plugin install"), tr("A more recent version of %1 search engine plugin is already installed.", "%1 is the name of the search engine").arg(plugin_name));
+    QMessageBox::information(this, tr("Search plugin install"), tr("A more recent version of '%1' search engine plugin is already installed.", "%1 is the name of the search engine").arg(plugin_name));
     return;
   }
   // Process with install
-  QString dest_path = fsutils::searchEngineLocation() + "/engines/" + plugin_name + ".py";
+  QString dest_path = Utils::Fs::searchEngineLocation() + "/engines/" + plugin_name + ".py";
   bool update = false;
   if (QFile::exists(dest_path)) {
     // Backup in case install fails
     QFile::copy(dest_path, dest_path+".bak");
-    fsutils::forceRemove(dest_path);
-    fsutils::forceRemove(dest_path+"c");
+    Utils::Fs::forceRemove(dest_path);
+    Utils::Fs::forceRemove(dest_path+"c");
     update = true;
   }
   // Copy the plugin
@@ -276,29 +288,29 @@ void engineSelectDlg::installPlugin(QString path, QString plugin_name) {
   if (!supported_engines->contains(plugin_name)) {
     if (update) {
       // Remove broken file
-      fsutils::forceRemove(dest_path);
+      Utils::Fs::forceRemove(dest_path);
       // restore backup
       QFile::copy(dest_path+".bak", dest_path);
-      fsutils::forceRemove(dest_path+".bak");
-      QMessageBox::warning(this, tr("Search plugin install"), tr("%1 search engine plugin could not be updated, keeping old version.", "%1 is the name of the search engine").arg(plugin_name));
+      Utils::Fs::forceRemove(dest_path+".bak");
+      QMessageBox::warning(this, tr("Search plugin install"), tr("'%1' search engine plugin could not be updated, keeping old version.", "%1 is the name of the search engine").arg(plugin_name));
       return;
     } else {
       // Remove broken file
-      fsutils::forceRemove(dest_path);
-      QMessageBox::warning(this, tr("Search plugin install"), tr("%1 search engine plugin could not be installed.", "%1 is the name of the search engine").arg(plugin_name));
+      Utils::Fs::forceRemove(dest_path);
+      QMessageBox::warning(this, tr("Search plugin install"), tr("'%1' search engine plugin could not be installed.", "%1 is the name of the search engine").arg(plugin_name));
       return;
     }
   }
   // Install was successful, remove backup and update plugin version
   if (update) {
-    fsutils::forceRemove(dest_path+".bak");
-    qreal version = SearchEngine::getPluginVersion(fsutils::searchEngineLocation() + "/engines/" + plugin_name + ".py");
+    Utils::Fs::forceRemove(dest_path+".bak");
+    qreal version = SearchEngine::getPluginVersion(Utils::Fs::searchEngineLocation() + "/engines/" + plugin_name + ".py");
     QTreeWidgetItem *item = findItemWithID(plugin_name);
     item->setText(ENGINE_VERSION, QString::number(version, 'f', 2));
-    QMessageBox::information(this, tr("Search plugin install"), tr("%1 search engine plugin was successfully updated.", "%1 is the name of the search engine").arg(plugin_name));
+    QMessageBox::information(this, tr("Search plugin install"), tr("'%1' search engine plugin was successfully updated.", "%1 is the name of the search engine").arg(plugin_name));
     return;
   } else {
-    QMessageBox::information(this, tr("Search plugin install"), tr("%1 search engine plugin was successfully installed.", "%1 is the name of the search engine").arg(plugin_name));
+    QMessageBox::information(this, tr("Search plugin install"), tr("'%1' search engine plugin was successfully installed.", "%1 is the name of the search engine").arg(plugin_name));
     return;
   }
 }
@@ -325,21 +337,21 @@ void engineSelectDlg::addNewEngine(QString engine_name) {
     setRowColor(pluginsTree->indexOfTopLevelItem(item), "red");
   }
   // Handle icon
-  QString iconPath = fsutils::searchEngineLocation() + "/engines/" + engine->getName() + ".png";
+  QString iconPath = Utils::Fs::searchEngineLocation() + "/engines/" + engine->getName() + ".png";
   if (QFile::exists(iconPath)) {
     // Good, we already have the icon
     item->setData(ENGINE_NAME, Qt::DecorationRole, QVariant(QIcon(iconPath)));
   } else {
-    iconPath = fsutils::searchEngineLocation() + "/engines/" + engine->getName() + ".ico";
+    iconPath = Utils::Fs::searchEngineLocation() + "/engines/" + engine->getName() + ".ico";
     if (QFile::exists(iconPath)) { // ICO support
       item->setData(ENGINE_NAME, Qt::DecorationRole, QVariant(QIcon(iconPath)));
     } else {
       // Icon is missing, we must download it
-      downloader->downloadUrl(engine->getUrl()+"/favicon.ico");
+      downloadFromUrl(engine->getUrl() + "/favicon.ico");
     }
   }
   // Load version
-  qreal version = SearchEngine::getPluginVersion(fsutils::searchEngineLocation() + "/engines/" + engine->getName() + ".py");
+  qreal version = SearchEngine::getPluginVersion(Utils::Fs::searchEngineLocation() + "/engines/" + engine->getName() + ".py");
   item->setText(ENGINE_VERSION, QString::number(version, 'f', 2));
 }
 
@@ -351,9 +363,16 @@ void engineSelectDlg::on_installButton_clicked() {
 
 void engineSelectDlg::askForPluginUrl() {
   bool ok(false);
+  QString clipTxt = qApp->clipboard()->text();
+  QString defaultUrl = "http://";
+  if ((clipTxt.startsWith("http://", Qt::CaseInsensitive)
+    || clipTxt.startsWith("https://", Qt::CaseInsensitive)
+    || clipTxt.startsWith("ftp://", Qt::CaseInsensitive))
+    && clipTxt.endsWith(".py"))
+    defaultUrl = clipTxt;
   QString url = AutoExpandableDialog::getText(this, tr("New search engine plugin URL"),
                                       tr("URL:"), QLineEdit::Normal,
-                                      "http://", &ok);
+                                      defaultUrl, &ok);
 
   while(true) {
     if (!ok || url.isEmpty())
@@ -369,7 +388,7 @@ void engineSelectDlg::askForPluginUrl() {
   }
 
   setCursor(QCursor(Qt::WaitCursor));
-  downloader->downloadUrl(url);
+  downloadFromUrl(url);
 }
 
 void engineSelectDlg::askForLocalPlugin() {
@@ -378,7 +397,7 @@ void engineSelectDlg::askForLocalPlugin() {
                                                         tr("qBittorrent search plugin")+QString::fromUtf8(" (*.py)"));
   foreach (QString path, pathsList) {
     if (path.endsWith(".py", Qt::CaseInsensitive)) {
-      QString plugin_name = fsutils::fileName(path);
+      QString plugin_name = Utils::Fs::fileName(path);
       plugin_name.chop(3); // Remove extension
       installPlugin(path, plugin_name);
     }
@@ -414,8 +433,8 @@ bool engineSelectDlg::parseVersionsFile(QString versions_file) {
       qDebug("Plugin: %s is outdated", qPrintable(plugin_name));
       // Downloading update
       setCursor(QCursor(Qt::WaitCursor));
-      downloader->downloadUrl(m_updateUrl + plugin_name + ".py");
-      //downloader->downloadUrl(m_updateUrl + plugin_name + ".png");
+      downloadFromUrl(m_updateUrl + plugin_name + ".py");
+      //downloadFromUrl(m_updateUrl + plugin_name + ".png");
       updated = true;
     }else {
       qDebug("Plugin: %s is up to date", qPrintable(plugin_name));
@@ -424,15 +443,22 @@ bool engineSelectDlg::parseVersionsFile(QString versions_file) {
   // Close file
   versions.close();
   // Clean up tmp file
-  fsutils::forceRemove(versions_file);
+  Utils::Fs::forceRemove(versions_file);
   if (file_correct && !updated) {
     QMessageBox::information(this, tr("Search plugin update"), tr("All your plugins are already up to date."));
   }
   return file_correct;
 }
 
-void engineSelectDlg::processDownloadedFile(QString url, QString filePath) {
-  filePath = fsutils::fromNativePath(filePath);
+void engineSelectDlg::downloadFromUrl(const QString &url)
+{
+    Net::DownloadHandler *handler = Net::DownloadManager::instance()->downloadUrl(url, true);
+    connect(handler, SIGNAL(downloadFinished(QString, QString)), this, SLOT(processDownloadedFile(QString, QString)));
+    connect(handler, SIGNAL(downloadFailed(QString, QString)), this, SLOT(handleDownloadFailure(QString, QString)));
+}
+
+void engineSelectDlg::processDownloadedFile(const QString &url, QString filePath) {
+  filePath = Utils::Fs::fromNativePath(filePath);
   setCursor(QCursor(Qt::ArrowCursor));
   qDebug("engineSelectDlg received %s", qPrintable(url));
   if (url.endsWith("favicon.ico", Qt::CaseInsensitive)) {
@@ -447,34 +473,34 @@ void engineSelectDlg::processDownloadedFile(QString url, QString filePath) {
         QFile icon(filePath);
         icon.open(QIODevice::ReadOnly);
         if (ICOHandler::canRead(&icon))
-          iconPath = fsutils::searchEngineLocation() + "/engines/" + id + ".ico";
+          iconPath = Utils::Fs::searchEngineLocation() + "/engines/" + id + ".ico";
         else
-          iconPath = fsutils::searchEngineLocation() + "/engines/" + id + ".png";
+          iconPath = Utils::Fs::searchEngineLocation() + "/engines/" + id + ".png";
         QFile::copy(filePath, iconPath);
         item->setData(ENGINE_NAME, Qt::DecorationRole, QVariant(QIcon(iconPath)));
       }
     }
     // Delete tmp file
-    fsutils::forceRemove(filePath);
+    Utils::Fs::forceRemove(filePath);
     return;
   }
   if (url.endsWith("versions.txt")) {
     if (!parseVersionsFile(filePath)) {
       QMessageBox::warning(this, tr("Search plugin update"), tr("Sorry, update server is temporarily unavailable."));
     }
-    fsutils::forceRemove(filePath);
+    Utils::Fs::forceRemove(filePath);
     return;
   }
   if (url.endsWith(".py", Qt::CaseInsensitive)) {
-    QString plugin_name = fsutils::fileName(url);
+    QString plugin_name = Utils::Fs::fileName(url);
     plugin_name.chop(3); // Remove extension
     installPlugin(filePath, plugin_name);
-    fsutils::forceRemove(filePath);
+    Utils::Fs::forceRemove(filePath);
     return;
   }
 }
 
-void engineSelectDlg::handleDownloadFailure(QString url, QString reason) {
+void engineSelectDlg::handleDownloadFailure(const QString &url, const QString &reason) {
   setCursor(QCursor(Qt::ArrowCursor));
   if (url.endsWith("favicon.ico", Qt::CaseInsensitive)) {
     qDebug("Could not download favicon: %s, reason: %s", qPrintable(url), qPrintable(reason));
@@ -488,6 +514,6 @@ void engineSelectDlg::handleDownloadFailure(QString url, QString reason) {
     // a plugin update download has been failed
     QString plugin_name = url.split('/').last();
     plugin_name.replace(".py", "", Qt::CaseInsensitive);
-    QMessageBox::warning(this, tr("Search plugin update"), tr("Sorry, %1 search plugin installation failed.", "%1 is the name of the search engine").arg(plugin_name));
+    QMessageBox::warning(this, tr("Search plugin update"), tr("Sorry, '%1' search plugin installation failed.", "%1 is the name of the search engine").arg(plugin_name));
   }
 }
