@@ -32,8 +32,16 @@
 #include <QObject>
 #include <QPointer>
 
-namespace Http { class Server; }
-class DNSUpdater;
+namespace Http
+{
+    class Server;
+}
+
+namespace Net
+{
+    class DNSUpdater;
+}
+
 class AbstractWebApplication;
 
 class WebUI : public QObject
@@ -48,8 +56,9 @@ private slots:
 
 private:
     QPointer<Http::Server> httpServer_;
-    QPointer<DNSUpdater> dynDNSUpdater_;
+    QPointer<Net::DNSUpdater> dynDNSUpdater_;
     QPointer<AbstractWebApplication> webapp_;
+    qint16 m_port;
 };
 
 #endif // WEBUI_H
